@@ -25,7 +25,7 @@
 
 *//****************************************************************//**
 
-\class TagsEditor
+\class TagsEditorDialog
 \brief Derived from ExpandingToolBar, this dialog allows editing of Tags.
 
 *//*******************************************************************/
@@ -44,18 +44,17 @@
 #endif
 
 #include "FileNames.h"
-#include "Internat.h"
 #include "Prefs.h"
+#include "Project.h"
+#include "ProjectFileIORegistry.h"
 #include "ShuttleGui.h"
-#include "TranslatableStringArray.h"
 #include "widgets/Grid.h"
-#include "widgets/ErrorDialog.h"
+#include "widgets/AudacityMessageBox.h"
 #include "widgets/HelpSystem.h"
 #include "xml/XMLFileReader.h"
 
 #include <wx/button.h>
 #include <wx/choice.h>
-#include <wx/filedlg.h>
 #include <wx/filename.h>
 #include <wx/intl.h>
 #include <wx/listctrl.h>
@@ -225,6 +224,32 @@ static const wxChar *DefaultGenres[] =
    wxT("Synthpop")
 };
 
+static ProjectFileIORegistry::Entry registerFactory{
+   wxT( "tags" ),
+   []( AudacityProject &project ){ return &Tags::Get( project ); }
+};
+
+static const AudacityProject::AttachedObjects::RegisteredFactory key{
+  [](AudacityProject &){ return std::make_shared< Tags >(); }
+};
+
+Tags &Tags::Get( AudacityProject &project )
+{
+   return project.AttachedObjects::Get< Tags >( key );
+}
+
+const Tags &Tags::Get( const AudacityProject &project )
+{
+   return Get( const_cast< AudacityProject & >( project ) );
+}
+
+Tags &Tags::Set( AudacityProject &project, const std::shared_ptr< Tags > &tags )
+{
+   auto &result = *tags;
+   project.AttachedObjects::Assign( key, tags );
+   return result;
+}
+
 Tags::Tags()
 {
    mEditTitle = true;
@@ -243,6 +268,13 @@ std::shared_ptr<Tags> Tags::Duplicate() const
    return std::make_shared<Tags>(*this);
 }
 
+void Tags::Merge( const Tags &other )
+{
+   for ( auto &pair : other.mMap ) {
+      SetTag( pair.first, pair.second );
+   }
+}
+
 Tags & Tags::operator=(const Tags & src)
 {
    mEditTitle = src.mEditTitle;
@@ -450,7 +482,7 @@ Tags::Iterators Tags::GetRange() const
    return { mMap.begin(), mMap.end() };
 }
 
-void Tags::SetTag(const wxString & name, const wxString & value)
+void Tags::SetTag(const wxString & name, const wxString & value, const bool bSpecialTag)
 {
    // We don't like empty names
    if (name.empty()) {
@@ -470,7 +502,10 @@ void Tags::SetTag(const wxString & name, const wxString & value)
    // Look it up
    TagMap::iterator iter = mXref.find(key);
 
-   if (value.empty()) {
+   // The special tags, if empty, should not exist.
+   // However it is allowable for a custom tag to be empty.
+   // See Bug 440 and Bug 1382 
+   if (value.empty() && bSpecialTag) {
       // Erase the tag
       if (iter == mXref.end())
          // nothing to do
@@ -480,7 +515,8 @@ void Tags::SetTag(const wxString & name, const wxString & value)
          mXref.erase(iter);
       }
    }
-   else {
+   else 
+   {
       if (iter == mXref.end()) {
          // Didn't find the tag
 
@@ -522,7 +558,7 @@ bool Tags::HandleXMLTag(const wxChar *tag, const wxChar **attrs)
          wxString value = *attrs++;
 
          if (!XMLValueChecker::IsGoodString(attr) ||
-             !XMLValueChecker::IsGoodString(value)) {
+             !XMLValueChecker::IsGoodLongString(value)) {
             break;
          }
 
@@ -577,10 +613,10 @@ void Tags::WriteXML(XMLWriter &xmlFile) const
    xmlFile.EndTag(wxT("tags"));
 }
 
-bool Tags::ShowEditDialog(wxWindow *parent, const wxString &title, bool force)
+bool Tags::ShowEditDialog(wxWindow *parent, const TranslatableString &title, bool force)
 {
    if (force) {
-      TagsEditor dlg(parent, title, this, mEditTitle, mEditTrackNumber);
+      TagsEditorDialog dlg(parent, title, this, mEditTitle, mEditTrackNumber);
 
       return dlg.ShowModal() == wxID_OK;
    }
@@ -704,37 +740,19 @@ private:
 #define LABEL_GENRE     XO("Genre")
 #define LABEL_COMMENTS  XO("Comments")
 
-static wxArrayString names()
-{
-   static wxString theNames[] =
-   {
-      LABEL_ARTIST,
-      LABEL_TITLE,
-      LABEL_ALBUM,
-      LABEL_TRACK,
-      LABEL_YEAR,
-      LABEL_GENRE,
-      LABEL_COMMENTS
-   };
-
-   class NamesArray final : public TranslatableStringArray
-   {
-      void Populate() override
-      {
-         for (auto &name : theNames)
-            mContents.push_back( wxGetTranslation( name ) );
-      }
-   };
-
-   static NamesArray theArray;
-
-   // Yes, return array by value
-   return theArray.Get();
-}
+static TranslatableStrings names{
+   LABEL_ARTIST,
+   LABEL_TITLE,
+   LABEL_ALBUM,
+   LABEL_TRACK,
+   LABEL_YEAR,
+   LABEL_GENRE,
+   LABEL_COMMENTS
+};
 
 static const struct
 {
-   wxString label;
+   TranslatableString label;
    wxString name;
 }
 labelmap[] =
@@ -762,25 +780,25 @@ enum {
    DontShowID
 };
 
-BEGIN_EVENT_TABLE(TagsEditor, wxDialogWrapper)
-   EVT_GRID_CELL_CHANGED(TagsEditor::OnChange)
-   EVT_BUTTON(EditID, TagsEditor::OnEdit)
-   EVT_BUTTON(ResetID, TagsEditor::OnReset)
-   EVT_BUTTON(ClearID, TagsEditor::OnClear)
-   EVT_BUTTON(LoadID, TagsEditor::OnLoad)
-   EVT_BUTTON(SaveID, TagsEditor::OnSave)
-   EVT_BUTTON(SaveDefaultsID, TagsEditor::OnSaveDefaults)
-   EVT_BUTTON(AddID, TagsEditor::OnAdd)
-   EVT_BUTTON(RemoveID, TagsEditor::OnRemove)
-   EVT_BUTTON(wxID_HELP, TagsEditor::OnHelp)
-   EVT_BUTTON(wxID_CANCEL, TagsEditor::OnCancel)
-   EVT_BUTTON(wxID_OK, TagsEditor::OnOk)
-   EVT_CHECKBOX( DontShowID, TagsEditor::OnDontShow )
-   EVT_KEY_DOWN(TagsEditor::OnKeyDown)
+BEGIN_EVENT_TABLE(TagsEditorDialog, wxDialogWrapper)
+   EVT_GRID_CELL_CHANGED(TagsEditorDialog::OnChange)
+   EVT_BUTTON(EditID, TagsEditorDialog::OnEdit)
+   EVT_BUTTON(ResetID, TagsEditorDialog::OnReset)
+   EVT_BUTTON(ClearID, TagsEditorDialog::OnClear)
+   EVT_BUTTON(LoadID, TagsEditorDialog::OnLoad)
+   EVT_BUTTON(SaveID, TagsEditorDialog::OnSave)
+   EVT_BUTTON(SaveDefaultsID, TagsEditorDialog::OnSaveDefaults)
+   EVT_BUTTON(AddID, TagsEditorDialog::OnAdd)
+   EVT_BUTTON(RemoveID, TagsEditorDialog::OnRemove)
+   EVT_BUTTON(wxID_HELP, TagsEditorDialog::OnHelp)
+   EVT_BUTTON(wxID_CANCEL, TagsEditorDialog::OnCancel)
+   EVT_BUTTON(wxID_OK, TagsEditorDialog::OnOk)
+   EVT_CHECKBOX( DontShowID, TagsEditorDialog::OnDontShow )
+   EVT_KEY_DOWN(TagsEditorDialog::OnKeyDown)
 END_EVENT_TABLE()
 
-TagsEditor::TagsEditor(wxWindow * parent,
-                       const wxString &title,
+TagsEditorDialog::TagsEditorDialog(wxWindow * parent,
+                       const TranslatableString &title,
                        Tags * tags,
                        bool editTitle,
                        bool editTrack)
@@ -790,7 +808,7 @@ TagsEditor::TagsEditor(wxWindow * parent,
    mEditTitle(editTitle),
    mEditTrack(editTrack)
 {
-   SetName(GetTitle());
+   SetName();
 
    mGrid = NULL;
 
@@ -814,10 +832,10 @@ TagsEditor::TagsEditor(wxWindow * parent,
 
    // Override size and position with last saved
    wxRect r = GetRect();
-   gPrefs->Read(wxT("/TagsEditor/x"), &r.x, r.x);
-   gPrefs->Read(wxT("/TagsEditor/y"), &r.y, r.y);
-   gPrefs->Read(wxT("/TagsEditor/width"), &r.width, r.width);
-   gPrefs->Read(wxT("/TagsEditor/height"), &r.height, r.height);
+   gPrefs->Read(wxT("/TagsEditorDialog/x"), &r.x, r.x);
+   gPrefs->Read(wxT("/TagsEditorDialog/y"), &r.y, r.y);
+   gPrefs->Read(wxT("/TagsEditorDialog/width"), &r.width, r.width);
+   gPrefs->Read(wxT("/TagsEditorDialog/height"), &r.height, r.height);
    //On multi-monitor systems, there's a chance the last saved window position is
    //on a monitor that has been removed or is unavailable.
    if (IsWindowRectValid(&r))
@@ -841,7 +859,7 @@ TagsEditor::TagsEditor(wxWindow * parent,
    PopulateGenres();
 }
 
-TagsEditor::~TagsEditor()
+TagsEditorDialog::~TagsEditorDialog()
 {
    // This DELETE is not needed because wxWidgets owns the grid.
 // DELETE mGrid;
@@ -854,7 +872,7 @@ TagsEditor::~TagsEditor()
 //   DELETE mComboEditor;
 }
 
-void TagsEditor::PopulateOrExchange(ShuttleGui & S)
+void TagsEditorDialog::PopulateOrExchange(ShuttleGui & S)
 {
    bool bShow;
    gPrefs->Read(wxT("/AudioFiles/ShowId3Dialog"), &bShow, true );
@@ -863,7 +881,7 @@ void TagsEditor::PopulateOrExchange(ShuttleGui & S)
    {
       S.StartHorizontalLay(wxALIGN_LEFT, 0);
       {
-         S.AddUnits(_("Use arrow keys (or ENTER key after editing) to navigate fields."));
+         S.AddUnits(XO("Use arrow keys (or ENTER key after editing) to navigate fields."));
       }
       S.EndHorizontalLay();
 
@@ -880,10 +898,11 @@ void TagsEditor::PopulateOrExchange(ShuttleGui & S)
 
          mGrid->SetColLabelSize(mGrid->GetDefaultRowSize());
 
-         auto  cs = names();
+         auto  cs = transform_container<wxArrayStringEx>(
+            names, std::mem_fn( &TranslatableString::Translation ) );
 
          // Build the initial (empty) grid
-         mGrid->CreateGrid(0, 2);
+         mGrid->CreateGrid(0, 2, wxGrid::wxGridSelectRows);
          mGrid->SetRowLabelSize(0);
          mGrid->SetDefaultCellAlignment(wxALIGN_LEFT, wxALIGN_CENTER);
          mGrid->SetColLabelValue(0, _("Tag"));
@@ -894,38 +913,39 @@ void TagsEditor::PopulateOrExchange(ShuttleGui & S)
          mGrid->SetColSize(0, tc.GetSize().x);
          mGrid->SetColMinimalWidth(0, tc.GetSize().x);
       }
-      S.Prop(1);
-      S.AddWindow(mGrid, wxEXPAND | wxALL);
+      S.Prop(1)
+         .Position(wxEXPAND | wxALL)
+         .AddWindow(mGrid);
 
       S.StartMultiColumn(4, wxALIGN_CENTER);
       {
-         S.Id(AddID).AddButton(_("&Add"));
-         S.Id(RemoveID).AddButton(_("&Remove"));
+         S.Id(AddID).AddButton(XXO("&Add"));
+         S.Id(RemoveID).AddButton(XXO("&Remove"));
          S.AddTitle( {} );
-         S.Id(ClearID).AddButton(_("Cl&ear"));
+         S.Id(ClearID).AddButton(XXO("Cl&ear"));
       }
       S.EndMultiColumn();
 
       S.StartHorizontalLay(wxALIGN_CENTRE, 0);
       {
-         S.StartStatic(_("Genres"));
+         S.StartStatic(XO("Genres"));
          {
             S.StartMultiColumn(4, wxALIGN_CENTER);
             {
-               S.Id(EditID).AddButton(_("E&dit..."));
-               S.Id(ResetID).AddButton(_("Rese&t..."));
+               S.Id(EditID).AddButton(XXO("E&dit..."));
+               S.Id(ResetID).AddButton(XXO("Rese&t..."));
             }
             S.EndMultiColumn();
          }
          S.EndStatic();
-         S.StartStatic(_("Template"));
+         S.StartStatic(XO("Template"));
          {
             S.StartMultiColumn(4, wxALIGN_CENTER);
             {
-               S.Id(LoadID).AddButton(_("&Load..."));
-               S.Id(SaveID).AddButton(_("&Save..."));
+               S.Id(LoadID).AddButton(XXO("&Load..."));
+               S.Id(SaveID).AddButton(XXO("&Save..."));
                S.AddTitle( {} );
-               S.Id(SaveDefaultsID).AddButton(_("Set De&fault"));
+               S.Id(SaveDefaultsID).AddButton(XXO("Set De&fault"));
             }
             S.EndMultiColumn();
          }
@@ -934,7 +954,7 @@ void TagsEditor::PopulateOrExchange(ShuttleGui & S)
       S.EndHorizontalLay();
       S.StartHorizontalLay(wxALIGN_LEFT, 0);
       {
-         S.Id( DontShowID ).AddCheckBox( _("Don't show this when exporting audio"), !bShow );
+         S.Id( DontShowID ).AddCheckBox( XXO("Don't show this when exporting audio"), !bShow );
       }
       S.EndHorizontalLay();
    }
@@ -943,19 +963,19 @@ void TagsEditor::PopulateOrExchange(ShuttleGui & S)
    S.AddStandardButtons(eOkButton | eCancelButton | eHelpButton);
 }
 
-void TagsEditor::OnDontShow( wxCommandEvent & Evt )
+void TagsEditorDialog::OnDontShow( wxCommandEvent & Evt )
 {
    bool bShow = !Evt.IsChecked();
    gPrefs->Write(wxT("/AudioFiles/ShowId3Dialog"), bShow );
    gPrefs->Flush();
 }
 
-void TagsEditor::OnHelp(wxCommandEvent& WXUNUSED(event))
+void TagsEditorDialog::OnHelp(wxCommandEvent& WXUNUSED(event))
 {
    HelpSystem::ShowHelp(this, wxT("Metadata_Editor"), true);
 }
 
-bool TagsEditor::TransferDataFromWindow()
+bool TagsEditorDialog::TransferDataFromWindow()
 {
    int i, cnt = mGrid->GetNumberRows();
 
@@ -968,43 +988,48 @@ bool TagsEditor::TransferDataFromWindow()
    for (i = 0; i < cnt; i++) {
       // Get tag name from the grid
 
-      wxString n = mGrid->GetCellValue(i, 0);
+      auto n = mGrid->GetCellValue(i, 0);
       wxString v = mGrid->GetCellValue(i, 1);
 
       if (n.empty()) {
          continue;
       }
 
+      bool bSpecialTag = true;
+
       // Map special tag names back to internal keys
-      if (n.CmpNoCase(wxGetTranslation(LABEL_ARTIST)) == 0) {
+      if (n.CmpNoCase(LABEL_ARTIST.Translation()) == 0) {
          n = TAG_ARTIST;
       }
-      else if (n.CmpNoCase(wxGetTranslation(LABEL_TITLE)) == 0) {
+      else if (n.CmpNoCase(LABEL_TITLE.Translation()) == 0) {
          n = TAG_TITLE;
       }
-      else if (n.CmpNoCase(wxGetTranslation(LABEL_ALBUM)) == 0) {
+      else if (n.CmpNoCase(LABEL_ALBUM.Translation()) == 0) {
          n = TAG_ALBUM;
       }
-      else if (n.CmpNoCase(wxGetTranslation(LABEL_TRACK)) == 0) {
+      else if (n.CmpNoCase(LABEL_TRACK.Translation()) == 0) {
          n = TAG_TRACK;
       }
-      else if (n.CmpNoCase(wxGetTranslation(LABEL_YEAR)) == 0) {
+      else if (n.CmpNoCase(LABEL_YEAR.Translation()) == 0) {
          n = TAG_YEAR;
       }
-      else if (n.CmpNoCase(wxGetTranslation(LABEL_GENRE)) == 0) {
+      else if (n.CmpNoCase(LABEL_GENRE.Translation()) == 0) {
          n = TAG_GENRE;
       }
-      else if (n.CmpNoCase(wxGetTranslation(LABEL_COMMENTS)) == 0) {
+      else if (n.CmpNoCase(LABEL_COMMENTS.Translation()) == 0) {
          n = TAG_COMMENTS;
       }
+      else {
+         bSpecialTag = false;
+      }
 
-      mLocal.SetTag(n, v);
+      mLocal.SetTag(n, v, bSpecialTag);
    }
 
    return true;
 }
 
-bool TagsEditor::TransferDataToWindow()
+bool TagsEditorDialog::TransferDataToWindow()
 {
    size_t i;
    TagMap popTagMap;
@@ -1024,16 +1049,16 @@ bool TagsEditor::TransferDataToWindow()
       mGrid->SetReadOnly(i, 0);
       // The special tag name that's displayed and translated may not match
       // the key string used for internal lookup.
-      mGrid->SetCellValue(i, 0, wxGetTranslation( labelmap[i].label ) );
+      mGrid->SetCellValue(i, 0, labelmap[i].label.Translation() );
       mGrid->SetCellValue(i, 1, mLocal.GetTag(labelmap[i].name));
 
       if (!mEditTitle &&
-          mGrid->GetCellValue(i, 0).CmpNoCase(wxGetTranslation(LABEL_TITLE)) == 0) {
+          mGrid->GetCellValue(i, 0).CmpNoCase(LABEL_TITLE.Translation()) == 0) {
          mGrid->SetReadOnly(i, 1);
       }
 
       if (!mEditTrack &&
-          mGrid->GetCellValue(i, 0).CmpNoCase(wxGetTranslation(LABEL_TRACK)) == 0) {
+          mGrid->GetCellValue(i, 0).CmpNoCase(LABEL_TRACK.Translation()) == 0) {
          mGrid->SetReadOnly(i, 1);
       }
 
@@ -1066,7 +1091,7 @@ bool TagsEditor::TransferDataToWindow()
    return true;
 }
 
-void TagsEditor::OnChange(wxGridEvent & event)
+void TagsEditorDialog::OnChange(wxGridEvent & event)
 {
    static bool ischanging = false;
 
@@ -1103,17 +1128,17 @@ void TagsEditor::OnChange(wxGridEvent & event)
    return;
 }
 
-void TagsEditor::OnEdit(wxCommandEvent & WXUNUSED(event))
+void TagsEditorDialog::OnEdit(wxCommandEvent & WXUNUSED(event))
 {
    if (mGrid->IsCellEditControlShown()) {
       mGrid->SaveEditControlValue();
       mGrid->HideCellEditControl();
    }
 
-   wxDialogWrapper dlg(this, wxID_ANY, _("Edit Genres"),
+   wxDialogWrapper dlg(this, wxID_ANY, XO("Edit Genres"),
                 wxDefaultPosition, wxDefaultSize,
                 wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);
-   dlg.SetName(dlg.GetTitle());
+   dlg.SetName();
    wxTextCtrl *tc;
 
    ShuttleGui S(&dlg, eIsCreating);
@@ -1145,7 +1170,9 @@ void TagsEditor::OnEdit(wxCommandEvent & WXUNUSED(event))
    wxFileName fn(FileNames::DataDir(), wxT("genres.txt"));
    wxFile f(fn.GetFullPath(), wxFile::write);
    if (!f.IsOpened() || !f.Write(tc->GetValue())) {
-      AudacityMessageBox(_("Unable to save genre file."), _("Reset Genres"));
+      AudacityMessageBox(
+         XO("Unable to save genre file."),
+         XO("Reset Genres") );
       return;
    }
 
@@ -1154,11 +1181,12 @@ void TagsEditor::OnEdit(wxCommandEvent & WXUNUSED(event))
    PopulateGenres();
 }
 
-void TagsEditor::OnReset(wxCommandEvent & WXUNUSED(event))
+void TagsEditorDialog::OnReset(wxCommandEvent & WXUNUSED(event))
 {
-   int id = AudacityMessageBox(_("Are you sure you want to reset the genre list to defaults?"),
-                         _("Reset Genres"),
-                         wxYES_NO);
+   int id = AudacityMessageBox(
+      XO("Are you sure you want to reset the genre list to defaults?"),
+      XO("Reset Genres"),
+      wxYES_NO);
 
    if (id == wxNO) {
       return;
@@ -1172,7 +1200,9 @@ void TagsEditor::OnReset(wxCommandEvent & WXUNUSED(event))
                (!tf.Exists() && tf.Create());
 
    if (!open) {
-      AudacityMessageBox(_("Unable to open genre file."), _("Reset Genres"));
+      AudacityMessageBox(
+         XO("Unable to open genre file."),
+         XO("Reset Genres") );
       mLocal.LoadGenres();
       return;
    }
@@ -1184,7 +1214,9 @@ void TagsEditor::OnReset(wxCommandEvent & WXUNUSED(event))
    }
 
    if (!tf.Write()) {
-      AudacityMessageBox(_("Unable to save genre file."), _("Reset Genres"));
+      AudacityMessageBox(
+         XO("Unable to save genre file."),
+         XO("Reset Genres") );
       mLocal.LoadGenres();
       return;
    }
@@ -1194,48 +1226,51 @@ void TagsEditor::OnReset(wxCommandEvent & WXUNUSED(event))
    PopulateGenres();
 }
 
-void TagsEditor::OnClear(wxCommandEvent & WXUNUSED(event))
+void TagsEditorDialog::OnClear(wxCommandEvent & WXUNUSED(event))
 {
    mLocal.Clear();
 
    TransferDataToWindow();
 }
 
-void TagsEditor::OnLoad(wxCommandEvent & WXUNUSED(event))
+void TagsEditorDialog::OnLoad(wxCommandEvent & WXUNUSED(event))
 {
    wxString fn;
 
    // Ask the user for the real name
    fn = FileNames::SelectFile(FileNames::Operation::_None,
-                     _("Load Metadata As:"),
-                     FileNames::DataDir(),
-                     wxT("Tags.xml"),
-                     wxT("xml"),
-                     wxT("*.xml"),
-                     wxFD_OPEN | wxRESIZE_BORDER,
-                     this);
+      XO("Load Metadata As:"),
+      FileNames::DataDir(),
+      wxT("Tags.xml"),
+      wxT("xml"),
+      { FileNames::XMLFiles },
+      wxFD_OPEN | wxRESIZE_BORDER,
+      this);
 
    // User canceled...
    if (fn.empty()) {
       return;
    }
 
+   // Load the metadata
+   decltype(mLocal) temp;
+   XMLFileReader reader;
+   if (!reader.Parse(&temp, fn)) {
+      // Inform user of load failure
+      AudacityMessageBox(
+         reader.GetErrorStr(),
+         XO("Error Loading Metadata"),
+         wxOK | wxCENTRE,
+         this);
+      return;
+   }
+
    // Remember title and track in case they're read only
    wxString title = mLocal.GetTag(TAG_TITLE);
    wxString track = mLocal.GetTag(TAG_TRACK);
 
-   // Clear current contents
-   mLocal.Clear();
-
-   // Load the metadata
-   XMLFileReader reader;
-   if (!reader.Parse(&mLocal, fn)) {
-      // Inform user of load failure
-      AudacityMessageBox(reader.GetErrorStr(),
-                   _("Error Loading Metadata"),
-                   wxOK | wxCENTRE,
-                   this);
-   }
+   // Replace existing tags with loaded ones
+   mLocal = temp;
 
    // Restore title
    if (!mEditTitle) {
@@ -1253,7 +1288,7 @@ void TagsEditor::OnLoad(wxCommandEvent & WXUNUSED(event))
    return;
 }
 
-void TagsEditor::OnSave(wxCommandEvent & WXUNUSED(event))
+void TagsEditorDialog::OnSave(wxCommandEvent & WXUNUSED(event))
 {
    wxString fn;
 
@@ -1262,13 +1297,13 @@ void TagsEditor::OnSave(wxCommandEvent & WXUNUSED(event))
 
    // Ask the user for the real name
    fn = FileNames::SelectFile(FileNames::Operation::_None,
-                     _("Save Metadata As:"),
-                     FileNames::DataDir(),
-                     wxT("Tags.xml"),
-                     wxT("xml"),
-                     wxT("*.xml"),
-                     wxFD_SAVE | wxFD_OVERWRITE_PROMPT | wxRESIZE_BORDER,
-                     this);
+      XO("Save Metadata As:"),
+      FileNames::DataDir(),
+      wxT("Tags.xml"),
+      wxT("xml"),
+      { FileNames::XMLFiles },
+      wxFD_SAVE | wxFD_OVERWRITE_PROMPT | wxRESIZE_BORDER,
+      this);
 
    // User canceled...
    if (fn.empty()) {
@@ -1277,7 +1312,7 @@ void TagsEditor::OnSave(wxCommandEvent & WXUNUSED(event))
 
    GuardedCall( [&] {
       // Create/Open the file
-      XMLFileWriter writer{ fn, _("Error Saving Tags File") };
+      XMLFileWriter writer{ fn, XO("Error Saving Tags File") };
 
       // Remember title and track in case they're read only
       wxString title = mLocal.GetTag(TAG_TITLE);
@@ -1312,7 +1347,7 @@ void TagsEditor::OnSave(wxCommandEvent & WXUNUSED(event))
    } );
 }
 
-void TagsEditor::OnSaveDefaults(wxCommandEvent & WXUNUSED(event))
+void TagsEditorDialog::OnSaveDefaults(wxCommandEvent & WXUNUSED(event))
 {
    // Refresh tags
    TransferDataFromWindow();
@@ -1353,22 +1388,22 @@ void TagsEditor::OnSaveDefaults(wxCommandEvent & WXUNUSED(event))
    }
 }
 
-void TagsEditor::OnAdd(wxCommandEvent & WXUNUSED(event))
+void TagsEditorDialog::OnAdd(wxCommandEvent & WXUNUSED(event))
 {
    mGrid->AppendRows();
 }
 
-void TagsEditor::OnRemove(wxCommandEvent & WXUNUSED(event))
+void TagsEditorDialog::OnRemove(wxCommandEvent & WXUNUSED(event))
 {
    size_t row = mGrid->GetGridCursorRow();
 
    if (!mEditTitle &&
-       mGrid->GetCellValue(row, 0).CmpNoCase(wxGetTranslation(LABEL_TITLE)) == 0) {
+       mGrid->GetCellValue(row, 0).CmpNoCase(LABEL_TITLE.Translation()) == 0) {
       return;
    }
    else if (!mEditTrack &&
             mGrid->GetCellValue(row, 0)
-               .CmpNoCase(wxGetTranslation(LABEL_TRACK)) == 0) {
+               .CmpNoCase(LABEL_TRACK.Translation()) == 0) {
       return;
    }
    else if (row < STATICCNT) {
@@ -1379,11 +1414,17 @@ void TagsEditor::OnRemove(wxCommandEvent & WXUNUSED(event))
    }
 }
 
-void TagsEditor::OnOk(wxCommandEvent & WXUNUSED(event))
+void TagsEditorDialog::OnOk(wxCommandEvent & WXUNUSED(event))
 {
    if (mGrid->IsCellEditControlShown()) {
       mGrid->SaveEditControlValue();
       mGrid->HideCellEditControl();
+#if defined(__WXMAC__)
+      // The cell editors do not capture the ENTER key, so it invokes
+      // the default button ("Ok") when it should just close the
+      // editor. So, cancel the "Ok" action.
+      return;
+#endif
    }
 
    if (!Validate() || !TransferDataFromWindow()) {
@@ -1393,21 +1434,21 @@ void TagsEditor::OnOk(wxCommandEvent & WXUNUSED(event))
    *mTags = mLocal;
 
    wxRect r = GetRect();
-   gPrefs->Write(wxT("/TagsEditor/x"), r.x);
-   gPrefs->Write(wxT("/TagsEditor/y"), r.y);
-   gPrefs->Write(wxT("/TagsEditor/width"), r.width);
-   gPrefs->Write(wxT("/TagsEditor/height"), r.height);
+   gPrefs->Write(wxT("/TagsEditorDialog/x"), r.x);
+   gPrefs->Write(wxT("/TagsEditorDialog/y"), r.y);
+   gPrefs->Write(wxT("/TagsEditorDialog/width"), r.width);
+   gPrefs->Write(wxT("/TagsEditorDialog/height"), r.height);
    gPrefs->Flush();
 
    EndModal(wxID_OK);
 }
 
-void TagsEditor::OnCancel(wxCommandEvent & WXUNUSED(event))
+void TagsEditorDialog::OnCancel(wxCommandEvent & WXUNUSED(event))
 {
    DoCancel(false);
 }
 
-void TagsEditor::DoCancel(bool escKey)
+void TagsEditorDialog::DoCancel(bool escKey)
 {
    if (mGrid->IsCellEditControlShown()) {
       auto editor = mGrid->GetCellEditor(mGrid->GetGridCursorRow(),
@@ -1428,7 +1469,7 @@ void TagsEditor::DoCancel(bool escKey)
    EndModal(wxID_CANCEL);
 }
 
-void TagsEditor::OnKeyDown(wxKeyEvent &event)
+void TagsEditorDialog::OnKeyDown(wxKeyEvent &event)
 {
    if (event.GetKeyCode() == WXK_ESCAPE)
       DoCancel(true);
@@ -1436,13 +1477,13 @@ void TagsEditor::OnKeyDown(wxKeyEvent &event)
       event.Skip();
 }
 
-void TagsEditor::SetEditors()
+void TagsEditorDialog::SetEditors()
 {
    int cnt = mGrid->GetNumberRows();
 
    for (int i = 0; i < cnt; i++) {
       wxString label = mGrid->GetCellValue(i, 0);
-      if (label.CmpNoCase(wxGetTranslation(LABEL_GENRE)) == 0) {
+      if (label.CmpNoCase(LABEL_GENRE.Translation()) == 0) {
          // This use of GetDefaultEditorForType does not require DecRef.
          mGrid->SetCellEditor(i, 1, mGrid->GetDefaultEditorForType(wxT("Combo")));
       }
@@ -1452,7 +1493,7 @@ void TagsEditor::SetEditors()
    }
 }
 
-void TagsEditor::PopulateGenres()
+void TagsEditorDialog::PopulateGenres()
 {
    int cnt = mLocal.GetNumUserGenres();
    int i;
@@ -1475,7 +1516,7 @@ void TagsEditor::PopulateGenres()
    editor->DecRef();
 }
 
-bool TagsEditor::IsWindowRectValid(const wxRect *windowRect) const
+bool TagsEditorDialog::IsWindowRectValid(const wxRect *windowRect) const
 {
    wxDisplay display;
    wxPoint topLeft(windowRect->GetTopLeft().x, windowRect->GetTopLeft().y);
